"""
Module containing concordia_class_search main functions.
"""

import datetime
import logging
import string

import requests

from . import exceptions
from . import scraping


URL = 'https://campus.concordia.ca/psc/pscsprd/EMPLOYEE/HRMS/c/CU_EXT.CU_CLASS_SEARCH.GBL'
logger = logging.getLogger(__name__)


def terms():
    r = requests.get(URL)
    terms = scraping.terms(r.text)
    terms.sort(key=lambda x: x['value'])
    return terms


def careers():
    r = requests.get(URL)
    careers = scraping.careers(r.text)
    careers.sort(key=lambda x: x['value'])
    return careers


def subjects():
    s = requests.Session()
    r = s.get(URL)
    data = scraping.data(r.text)
    data['ICAction'] = 'CLASS_SRCH_WRK2_SSR_PB_SUBJ_SRCH$1'
    r = s.post(URL, data=data)
    subjects = []
    for char in (string.ascii_uppercase + string.digits):
        data = scraping.data(r.text)
        data['ICAction'] = 'SSR_CLSRCH_WRK2_SSR_ALPHANUM_' + char
        r = s.post(URL, data=data)
        subjects_char = scraping.subjects(r.text)
        subjects.extend(subjects_char)
    subjects.sort(key=lambda x: x['value'])
    return subjects


def class_search(term, career, subject, number=None, details=False):
    s = requests.Session()
    r = s.get(URL)
    data = scraping.data(r.text)

    # get class term
    terms = scraping.terms(r.text)
    for t in terms:
        if t['value'] == int(term):
            term = t
            break
    if not isinstance(term, dict):
        raise exceptions.InvalidTerm(f'invalid term: {term!r}')

    # get class career
    careers = scraping.careers(r.text)
    for c in careers:
        if c['value'] == career.upper():
            career = c
            break
    if not isinstance(career, dict):
        raise exceptions.InvalidCareer(f'invalid career: {career!r}')

    # get class subject
    data['ICAction'] = 'SSR_CLSRCH_WRK_SUBJECT$1'
    data['SSR_CLSRCH_WRK_SUBJECT$1'] = subject
    r = s.post(URL, data=data)
    subject_text = scraping.subject_text(r.text)
    if not subject_text:
        raise exceptions.InvalidSubject(f'invalid subject: {subject!r}')
    logger.info(f'found subject: {subject!r} -> {subject_text!r}')
    subject = {'value': subject.upper(), 'text': subject_text}

    if number: number = int(number)

    class_search = {
        'datetime': datetime.datetime.now(datetime.timezone.utc).replace(microsecond=0).isoformat(),
        'term': term,
        'career': career,
        'subject': subject,
        'number': number,
    }

    data = scraping.data(r.text)
    data['ICAction'] = 'CLASS_SRCH_WRK2_SSR_PB_CLASS_SRCH'
    data['CLASS_SRCH_WRK2_STRM$35$'] = term['value']
    data['SSR_CLSRCH_WRK_ACAD_CAREER$0'] = career['value']
    if number: data['SSR_CLSRCH_WRK_CATALOG_NBR$2'] = number

    logger.debug('sending request with data: {}'.format(data))
    r = s.post(URL, data=data)

    # handle prompts
    prompts = scraping.prompts(r.text)

    error = prompts.get('error')
    if error:
        logger.error(f'error prompt: {error!r}')
        if 'maximum limit of 300 sections' in error:
            raise exceptions.TooManySections(error)
        if 'search returns no results' in error:
            raise exceptions.NoResults(error)
        else:
            raise exceptions.UnknownError(error)

    info = prompts.get('info')
    if info:
        logger.info(f'info prompt: {info!r}')
        data = scraping.data(r.text)
        data['ICAction'] = '#ICSave'
        logger.debug('sending request with data: {}'.format(data))
        r = s.post(URL, data=data)

    classes = scraping.classes(r.text)
    for class_i in range(len(classes)):
        for section_i in range(len(classes[class_i]['sections'])):

            if details:
                logging.debug('getting details for section number {}'.format(classes[class_i]['sections'][section_i]['number']))
                data = scraping.data(r.text)
                data['ICAction'] = classes[class_i]['sections'][section_i]['_class_number_id']
                r = s.post(URL, data=data)
                section_details = scraping.section_details(r.text)
                classes[class_i]['sections'][section_i].update(section_details)
                data = scraping.data(r.text)
                data['ICAction'] = 'CLASS_SRCH_WRK2_SSR_PB_BACK'
                r = s.post(URL, data=data)

            del classes[class_i]['sections'][section_i]['_class_number_id']

            section_term_text = classes[class_i]['sections'][section_i]['term']
            section_term = next(filter(lambda x: x['text'] == section_term_text, terms))
            classes[class_i]['sections'][section_i]['term'] = section_term

        classes[class_i]['subject'] = subject

    results = {
        'class_search': class_search,
        'classes': classes,
    }

    return results
