"""
Module contining Concordia Class Search scraping functions.
"""

import logging
import re

import bs4


logger = logging.getLogger(__name__)


def _normalize_string(s):
    s = s.strip()
    s = s.replace('\r', ' ')
    s = s.replace('\xa0', ' ')
    s = ' '.join(s.split())
    return s


def _normalize_soup(soup):
    for cdata_id in ['win0divPSHIDDENFIELDS', 'win0divPAGECONTAINER']:
        element = soup.find(id=cdata_id)
        if element and type(element.string) is bs4.element.CData:
            logger.debug(f'replacing cdata with id: {cdata_id}')
            sub_soup = bs4.BeautifulSoup(element.text, 'html.parser')
            element.string = ''
            for tag in sub_soup:
                element.append(tag)


def _get_soup(html):
    soup = bs4.BeautifulSoup(html, 'html.parser')
    _normalize_soup(soup)
    return soup


def data(html):
    soup = _get_soup(html)
    data = {
        'ICSID': soup.find(id='ICSID')['value'],
        'ICStateNum': soup.find(id='ICStateNum')['value'],
    }
    return data


def terms(html):
    soup = _get_soup(html)
    terms = []
    for term in soup.find(id='CLASS_SRCH_WRK2_STRM$35$').find_all('option'):
        if term['value']:
            terms.append({
                'value': int(term['value']),
                'text': _normalize_string(term.text),
            })
    return terms


def careers(html):
    soup = _get_soup(html)
    careers = []
    for career in soup.find(id='SSR_CLSRCH_WRK_ACAD_CAREER$0').find_all('option'):
        if career['value']:
            careers.append({
                'value': _normalize_string(career['value']).upper(),
                'text': _normalize_string(career.text),
            })
    return careers


def subject_text(html):
    soup = _get_soup(html)
    return _normalize_string(soup.find(id='SUBJECT_TBL_DESCR$1').text)


def subjects(html):
    soup = _get_soup(html)
    val = [subj.text for subj in soup.find_all(id=re.compile('^SSR_CLSRCH_SUBJ_SUBJECT\$\d+$'))]
    tex = [subj.text for subj in soup.find_all(id=re.compile('^SUBJECT_TBL_DESCRFORMAL\$\d+$'))]
    subjects = []
    for value, text in zip(val, tex):
        subjects.append({
            'value': _normalize_string(value).upper(),
            'text': _normalize_string(text)
        })
    return subjects


def prompts(html):
    soup = _get_soup(html)
    prompts = {}
    error = soup.find(id='DERIVED_CLSMSG_ERROR_TEXT')
    info = soup.find(id='DERIVED_SSE_DSP_SSR_MSG_TEXT')
    if error:
        prompts['error'] = _normalize_string(error.text)
    if info:
        prompts['info'] = _normalize_string(info.text)
    return prompts


def _element_text(soup, element_id, function=str, required=False):
    element = soup.find(id=element_id)
    if element:
        return function(_normalize_string(element.text))
    elif required:
        raise ValueError('required element not found: {}'.format(element_id))
    else:
        return None


def _class_section(section_soup):
    status = section_soup.find(id=re.compile('^win0divDERIVED_CLSRCH_SSR_STATUS_LONG\$\d+$'))
    status = _normalize_string(status.find('img')['alt'])

    return {
        'number': _element_text(section_soup, re.compile(r'^MTG_CLASS_NBR\$\d+$'), function=int, required=True),
        'section': _element_text(section_soup, re.compile(r'^MTG_CLASSNAME\$\d+$'), required=True),
        'daytime': _element_text(section_soup, re.compile(r'^MTG_DAYTIME\$\d+$'), required=True),
        'room': _element_text(section_soup, re.compile(r'^MTG_ROOM\$\d+$'), required=True),
        'instructor': _element_text(section_soup, re.compile(r'^MTG_INSTR\$\d+$'), required=True),
        'dates': _element_text(section_soup, re.compile(r'^MTG_TOPIC\$\d+$'), required=True),
        'term': _element_text(section_soup, re.compile(r'^SSR_CSTRMADM_VW_DESCR\$\d+$'), required=True),
        'status': status,

        # needed to get section details later
        '_class_number_id': section_soup.find(id=re.compile(r'^MTG_CLASSNAME\$\d+$'))['id'],
    }


def _class_sections(class_soup):
    sections = []
    for section_soup in class_soup.find_all(id=re.compile(r'^win0divSSR_CLSRSLT_WRK_GROUPBOX3\$\d+$')):
        section = _class_section(section_soup)
        sections.append(section)
    return sections


def _class_info(class_soup):
    title = class_soup.find('td', class_='PAGROUPBOXLABELLEVEL1 PSLEFTCORNER').text
    title = ' '.join(title.split())
    m = re.match('^([a-zA-Z]{3,4}) ([0-9a-zA-Z]{0,5}) \- (.+)$', title)
    subject = m.group(1).upper()
    number = m.group(2)
    if number.isdigit():
        number = int(number)
    else:
        number = number.upper()
    name = m.group(3)
    return {
        'subject': subject,
        'number': number,
        'name': name,
    }


def classes(html):
    soup = _get_soup(html)
    classes = []
    for class_soup in soup.find_all(id=re.compile('^win0divSSR_CLSRSLT_WRK_GROUPBOX2\$\d+$')):
        class_ = _class_info(class_soup)
        class_['sections'] = _class_sections(class_soup)
        logger.info('found {0:02d} section(s) for class: "{1} {2} - {3}"'.format(
            len(class_['sections']), class_['subject'], class_['number'], class_['name']))
        classes.append(class_)
    logger.info('returning {} class(es)'.format(len(classes)))
    return classes


def section_details(html):
    soup = _get_soup(html)

    section_details = {
        # class details
        'status': _element_text(soup, 'SSR_CLS_DTL_WRK_SSR_DESCRSHORT'),
        'class_number': _element_text(soup, 'SSR_CLS_DTL_WRK_CLASS_NBR', int),
        'session': _element_text(soup, 'PSXLATITEM_XLATLONGNAME$31$'),
        'units': _element_text(soup, 'SSR_CLS_DTL_WRK_UNITS_RANGE'),
        'instruction_mode': _element_text(soup, 'INSTRUCT_MODE_DESCR'),
        'class_components': _element_text(soup, 'win0divSSR_CLS_DTL_WRK_SSR_COMPONENT_LONG'),

        'career': _element_text(soup, 'PSXLATITEM_XLATLONGNAME'),
        'dates': _element_text(soup, 'SSR_CLS_DTL_WRK_SSR_DATE_LONG'),
        'grading': _element_text(soup, 'GRADE_BASIS_TBL_DESCRFORMAL'),
        'location': _element_text(soup, 'CAMPUS_LOC_VW_DESCR'),
        'campus': _element_text(soup, 'CAMPUS_TBL_DESCR'),

        # class availability
        'class_capacity': _element_text(soup, 'SSR_CLS_DTL_WRK_ENRL_CAP', int),
        'enrollment_total': _element_text(soup, 'SSR_CLS_DTL_WRK_ENRL_TOT', int),
        'available_seats': _element_text(soup, 'SSR_CLS_DTL_WRK_AVAILABLE_SEATS', int),
        'wait_list_capacity': _element_text(soup, 'SSR_CLS_DTL_WRK_WAIT_CAP', int),
        'wait_list_total': _element_text(soup, 'SSR_CLS_DTL_WRK_WAIT_TOT', int),

        # notes
        'class_notes': _element_text(soup, 'DERIVED_CLSRCH_SSR_CLASSNOTE_LONG'),

        # description
        'description': _element_text(soup, 'DERIVED_CLSRCH_DESCRLONG'),

        # textbook/other materials
        'textbook': _element_text(soup, 'SSR_CLS_DTL_WRK_SSR_CLS_TXB_MSG'),
    }

    return section_details
